#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include "uart.h"

#if defined(__AVR_ATmega8__)
#else
#  error "Don't know what kind of MCU you are compiling for!"
#endif

#define XTAL_CPU         4000000      /* 4MHz */
#define UART_BAUD_RATE      4800      /* 9600 baud */

enum { ON, HALF, OFF };

volatile uint8_t status;

SIGNAL(SIG_INTERRUPT0)
{
    switch (status)
    {
        case ON:
			uart_puts("HALF\n\r");
			outp (0x01,PORTC);
            status = HALF;
            break;

        case HALF:
			uart_puts("OFF\n\r");
			outp (0xFF,PORTC);
            status = OFF;
            break;

        case OFF:
			uart_puts("ON\n\r");
			outp (0x00,PORTC);
            status = ON;
            break;
    }
}

void ioinit (void)
{
	// Port C ist OUTPUT
	outp( 0xFF, DDRC );
	// Aktiviere externen Interrupt 0
	outp( ( 1 << INT0), GIMSK );
	// Externer Interrupt 0 bei fallender Flanke
	outp( ( 1 << ISC01) | ( 1 << ISC11 ), MCUCR );
	// Initialisiere LEDs: AN
	outp( 0x00, PORTC );
	status=ON;
}

int main (void)
{
	// Zeichen fuer den Empfang
    unsigned int c;
	unsigned char echo;

	// Initialisierung(en) durchfuehren
    ioinit ();
    /*
     *  Initialize UART library, pass baudrate and avr cpu clock 
     *  with the macro UART_BAUD_SELECT()
     */
    uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,XTAL_CPU) ); 
    // Aktiviere Interruptverarbeitung
    sei ();
	
	echo=1;

    /*
     *  Transmit string to UART
     *  The string is buffered by the uart library in a circular buffer
     *  and one character at a time is transmitted to the UART using interrupts.
     *  uart_puts() blocks if it can not write the whole string to the circular 
     *  buffer
     */
    uart_puts("String stored in SRAM\n\r");
    /*
     * Transmit string from program memory to UART
     */
    uart_puts_P("String stored in FLASH\n\r");
    /*
     * Transmit single character to UART
     */
	uart_putc('\n');
    uart_putc('\r');

	for(;;)
	{
        /*
         * Get received character from ringbuffer
         * uart_getc() returns in the lower byte the received character and 
         * in the higher byte (bitmask) the last receive error
         * UART_NO_DATA is returned when no data is available.
         *
         */
        c = uart_getc();
        if ( c & UART_NO_DATA )
        {
            /* 
             * no data available from UART 
             */
        }
        else
        {
		if(echo) uart_putc('\n');
		if(echo) uart_putc('\r');
            /*
             * new data available from UART
             * check for Frame or Overrun error
             */
            if ( c & UART_FRAME_ERROR )
            {
                /* Framing Error detected, i.e no stop bit detected */
                uart_puts_P("UART Frame Error: ");
            }
            if ( c & UART_OVERRUN_ERROR )
            {
                /* 
                 * Overrun, a character already present in the UART UDR register was 
                 * not read by the interrupt handler before the next character arrived,
                 * one or more received characters have been dropped
                 */
                uart_puts_P("UART Overrun Error: ");
            }
            if ( c & UART_BUFFER_OVERFLOW )
            {
                /* 
                 * We are not reading the receive buffer fast enough,
                 * one or more received character have been dropped 
                 */
                uart_puts_P("Buffer overflow error: ");
            }
            /* 
             * send received character back
             */
            uart_putc( (unsigned char)c );
        }
	}
	return (0);
}
